@@ -36,9 +36,14 @@ extern gboolean ugly_line_drawing;
 /* The mouse is currently: TRUE - enabled, FALSE - disabled */
 extern gboolean mouse_enabled;
 
+/* terminal ca capabilities */
+extern char *smcup;
+extern char *rmcup;
+
 /*** declarations of public functions ************************************************************/
 
 char *mc_tty_normalize_from_utf8 (const char *);
+void tty_init_xterm_support (gboolean is_xterm);
 
 /*** inline functions ****************************************************************************/
 #endif /* MC_TTY_INTERNAL_H */
