@@ -159,7 +159,7 @@ slang_reset_softkeys (void)
         send = (char *) SLtt_tgetstr (tmp);
         if (send != NULL)
         {
-            g_snprintf (tmp, sizeof (tmp), "\033&f%dk%dd%dL%s%s", key,
+            g_snprintf (tmp, sizeof (tmp), ESC_STR "&f%dk%dd%dL%s%s", key,
                         (int) (sizeof (display) - 1), (int) strlen (send), display, send);
             SLtt_write_string (tmp);
         }
@@ -241,14 +241,14 @@ mc_tty_normalize_lines_char (const char *str)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-tty_init (gboolean slow, gboolean ugly_lines)
+tty_init (gboolean slow, gboolean ugly_lines, gboolean mouse_enable, gboolean is_xterm)
 {
     slow_tty = slow;
     ugly_line_drawing = ugly_lines;
 
     SLtt_Ignore_Beep = 1;
 
-    SLutf8_enable (-1); /* has to be called first before any of the other functions. */
+    SLutf8_enable (-1);         /* has to be called first before any of the other functions. */
     SLtt_get_terminfo ();
     /*
      * If the terminal in not in terminfo but begins with a well-known
@@ -300,7 +300,18 @@ tty_init (gboolean slow, gboolean ugly_lines)
     /* It's the small part from the previous init_key() */
     init_key_input_fd ();
 
+    /* For 8-bit locales, NCurses handles 154 (0x9A) symbol properly, while S-Lang
+     * requires SLsmg_Display_Eight_Bit >= 154 (OR manual filtering if xterm display
+     * detected - but checking TERM would fail under screen, OR running xterm
+     * with allowC1Printable).
+     */
+    tty_display_8bit (FALSE);
+
     SLsmg_init_smg ();
+    if (!mouse_enable)
+        use_mouse_p = MOUSE_DISABLED;
+    tty_init_xterm_support (is_xterm);  /* do it before do_enter_ca_mode() call */
+    init_mouse ();
     do_enter_ca_mode ();
     tty_keypad (TRUE);
     tty_nodelay (FALSE);
@@ -313,8 +324,11 @@ tty_shutdown (void)
 {
     char *op_cap;
 
-    SLsmg_reset_smg ();
+    disable_mouse ();
     tty_reset_shell_mode ();
+    tty_noraw_mode ();
+    tty_keypad (FALSE);
+    tty_reset_screen ();
     do_exit_ca_mode ();
     SLang_reset_tty ();
 
