@@ -1468,3 +1468,28 @@ vfs_s_get_line_interruptible (struct vfs_class *me, char *buffer, int size, int
 #endif /* ENABLE_VFS_NET */
 
 /* --------------------------------------------------------------------------------------------- */
+
+/**
+ * Normalize filenames start position
+ */
+
+void
+vfs_s_normalize_filename_leading_spaces (struct vfs_s_inode *root_inode, size_t final_num_spaces)
+{
+    struct vfs_s_entry *entry;
+
+    for (entry = root_inode->subdir; entry != NULL; entry = entry->next)
+    {
+        if ((size_t) entry->ino->data_offset > final_num_spaces)
+        {
+            char *source_name = entry->name;
+            char *spacer = g_strnfill (entry->ino->data_offset - final_num_spaces, ' ');
+            entry->name = g_strdup_printf ("%s%s", spacer, source_name);
+            g_free (spacer);
+            g_free (source_name);
+        }
+        entry->ino->data_offset = -1;
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
