@@ -275,6 +275,7 @@ fish_free_archive (struct vfs_class *me, struct vfs_s_super *super)
     g_free (SUP.scr_unlink);
     g_free (SUP.scr_chown);
     g_free (SUP.scr_chmod);
+    g_free (SUP.scr_utime);
     g_free (SUP.scr_rmdir);
     g_free (SUP.scr_ln);
     g_free (SUP.scr_mv);
@@ -575,6 +576,7 @@ fish_open_archive (struct vfs_class *me, struct vfs_s_super *super,
     SUP.scr_unlink = fish_load_script_from_file (host, FISH_UNLINK_FILE, FISH_UNLINK_DEF_CONTENT);
     SUP.scr_chown = fish_load_script_from_file (host, FISH_CHOWN_FILE, FISH_CHOWN_DEF_CONTENT);
     SUP.scr_chmod = fish_load_script_from_file (host, FISH_CHMOD_FILE, FISH_CHMOD_DEF_CONTENT);
+    SUP.scr_utime = fish_load_script_from_file (host, FISH_UTIME_FILE, FISH_UTIME_DEF_CONTENT);
     SUP.scr_rmdir = fish_load_script_from_file (host, FISH_RMDIR_FILE, FISH_RMDIR_DEF_CONTENT);
     SUP.scr_ln = fish_load_script_from_file (host, FISH_LN_FILE, FISH_LN_DEF_CONTENT);
     SUP.scr_mv = fish_load_script_from_file (host, FISH_MV_FILE, FISH_MV_DEF_CONTENT);
@@ -1271,6 +1273,37 @@ fish_chown (struct vfs_class *me, const char *path, uid_t owner, gid_t group)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static int
+fish_utime (struct vfs_class *me, const char *path, struct utimbuf *times)
+{
+    gchar *shell_commands = NULL;
+    char utcmtime[16], utcatime[16];
+    struct tm *gmt;
+
+    PREFIX;
+
+    gmt = gmtime(&times->modtime);
+    g_snprintf (utcmtime, sizeof (utcmtime), "%04d%02d%02d%02d%02d.%02d",
+                gmt->tm_year+1900, gmt->tm_mon+1, gmt->tm_mday,
+                gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
+
+    gmt = gmtime(&times->actime);
+    g_snprintf (utcatime, sizeof (utcatime), "%04d%02d%02d%02d%02d.%02d",
+                gmt->tm_year+1900, gmt->tm_mon+1, gmt->tm_mday,
+                gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
+
+    shell_commands = g_strconcat (SUP.scr_env, "FISH_FILENAME=%s FISH_FILEATIME=%ld FISH_FILEMTIME=%ld ",
+                                  "FISH_TOUCHATIME=%s FISH_TOUCHMTIME=%s;\n",
+                                  SUP.scr_utime, (char *) NULL);
+    g_snprintf (buf, sizeof (buf), shell_commands,
+                rpath, (long)times->actime, (long)times->modtime, utcatime, utcmtime);
+    g_free (shell_commands);
+    g_free (rpath);
+    return fish_send_command (me, super, buf, OPT_FLUSH);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 fish_unlink (struct vfs_class *me, const char *path)
 {
@@ -1458,6 +1491,7 @@ init_fish (void)
     vfs_fish_ops.fill_names = fish_fill_names;
     vfs_fish_ops.chmod = fish_chmod;
     vfs_fish_ops.chown = fish_chown;
+    vfs_fish_ops.utime = fish_utime;
     vfs_fish_ops.open = fish_open;
     vfs_fish_ops.symlink = fish_symlink;
     vfs_fish_ops.link = fish_link;
