@@ -592,6 +592,8 @@ ftpfs_login_server (struct vfs_class *me, struct vfs_s_super *super, const char
 
         reply_up = g_ascii_strup (reply_string, -1);
         SUP.remote_is_amiga = strstr (reply_up, "AMIGA") != 0;
+        if (strstr (reply_up, " SPFTP/1.0.0000 SERVER ")) /* handles `LIST -la` in a weird way */
+            SUP.strict = RFC_STRICT;
         g_free (reply_up);
 
         if (MEDATA->logfile)
@@ -1687,9 +1689,11 @@ ftpfs_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path
     /* Clear the interrupt flag */
     tty_enable_interrupt_key ();
 
+    vfs_parse_ls_lga_init ();
     while (1)
     {
         int i;
+        size_t count_spaces = 0;
         int res = vfs_s_get_line_interruptible (me, lc_buffer, sizeof (lc_buffer),
                                                 sock);
         if (!res)
@@ -1714,13 +1718,15 @@ ftpfs_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path
 
         ent = vfs_s_generate_entry (me, NULL, dir, 0);
         i = ent->ino->st.st_nlink;
-        if (!vfs_parse_ls_lga (lc_buffer, &ent->ino->st, &ent->name, &ent->ino->linkname))
+        if (!vfs_parse_ls_lga
+            (lc_buffer, &ent->ino->st, &ent->name, &ent->ino->linkname, &count_spaces))
         {
             vfs_s_free_entry (me, ent);
             continue;
         }
         ent->ino->st.st_nlink = i;      /* Ouch, we need to preserve our counts :-( */
         num_entries++;
+        vfs_s_store_filename_leading_spaces (ent, count_spaces);
         vfs_s_insert_entry (me, dir, ent);
     }
 
@@ -1745,6 +1751,8 @@ ftpfs_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path
         goto again;
     }
 
+    vfs_s_normalize_filename_leading_spaces (dir, vfs_parse_ls_lga_get_final_spaces ());
+
     if (SUP.strict == RFC_AUTODETECT)
         SUP.strict = RFC_DARING;
 
