@@ -61,6 +61,17 @@ get_hotkey (int n)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static void
+dialog_switch_suspend (void *data, void *user_data)
+{
+    (void) user_data;
+
+    if (data != mc_current->data)
+        ((Dlg_head *) data)->state = DLG_SUSPENDED;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 dialog_switch_goto (GList * dlg)
 {
@@ -86,8 +97,11 @@ dialog_switch_goto (GList * dlg)
                 /* return to panels before run the required dialog */
                 dialog_switch_pending = TRUE;
             else
+            {
                 /* switch to panels */
+                midnight_dlg->state = DLG_ACTIVE;
                 do_refresh ();
+            }
         }
     }
 }
@@ -110,6 +124,9 @@ dialog_switch_add (Dlg_head * h)
         mc_dialogs = g_list_prepend (mc_dialogs, h);
         mc_current = mc_dialogs;
     }
+
+    /* suspend forced all other screens */
+    g_list_foreach (mc_dialogs, dialog_switch_suspend, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -131,6 +148,10 @@ dialog_switch_remove (Dlg_head * h)
         mc_current = g_list_find (mc_dialogs, (Dlg_head *) top_dlg->data);
     else
         mc_current = mc_dialogs;
+
+    /* resume forced the current screen */
+    if (mc_current != NULL)
+        ((Dlg_head *) mc_current->data)->state = DLG_ACTIVE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -239,6 +260,7 @@ dialog_switch_process_pending (void)
         if (h->state == DLG_CLOSED)
         {
             destroy_dlg (h);
+
             /* return to panels */
             if (mc_run_mode == MC_RUN_FULL)
             {
