@@ -196,6 +196,13 @@ static const GOptionEntry argument_terminal_table[] = {
      NULL
     },
 
+    {
+     "oldmouse", 'g', ARGS_TERM_OPTIONS, G_OPTION_ARG_NONE,
+     &old_mouse,
+     N_("Tries to use an old highlight mouse tracking"),
+     NULL
+    },
+
     {
      "nomouse", 'd', ARGS_TERM_OPTIONS, G_OPTION_ARG_NONE,
      &mc_args__nomouse,
@@ -388,8 +395,10 @@ mc_setup_by_args (int argc, char *argv[])
     const char *base;
     char *tmp;
 
-    if (mc_args__nomouse)
-        use_mouse_p = MOUSE_DISABLED;
+#ifdef ENABLE_VFS_SMB
+    if (mc_args__debug_level != 0)
+        smbfs_set_debug (mc_args__debug_level);
+#endif /* ENABLE_VFS_SMB */
 
     if (mc_args__netfs_logfile != NULL)
     {
@@ -397,15 +406,10 @@ mc_setup_by_args (int argc, char *argv[])
         mc_setctl ("/#ftp:", VFS_SETCTL_LOGFILE, (void *) mc_args__netfs_logfile);
 #endif /* ENABLE_VFS_FTP */
 #ifdef ENABLE_VFS_SMB
-        smbfs_set_debugf (mc_args__netfs_logfile);
+        mc_setctl ("/#smb:", VFS_SETCTL_LOGFILE, (void *) mc_args__netfs_logfile);
 #endif /* ENABLE_VFS_SMB */
     }
 
-#ifdef ENABLE_VFS_SMB
-    if (mc_args__debug_level != 0)
-        smbfs_set_debug (mc_args__debug_level);
-#endif /* ENABLE_VFS_SMB */
-
     base = x_basename (argv[0]);
     tmp = (argc > 0) ? argv[1] : NULL;
 
