@@ -39,16 +39,6 @@
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
-#define SEARCH_DIALOG_OPTION_NO_SCANF   (1 << 0)
-#define SEARCH_DIALOG_OPTION_NO_REGEX   (1 << 1)
-#define SEARCH_DIALOG_OPTION_NO_CASE    (1 << 2)
-#define SEARCH_DIALOG_OPTION_BACKWARDS  (1 << 3)
-#define SEARCH_DIALOG_OPTION_BOOKMARK   (1 << 4)
-
-#define EDIT_KEY_EMULATION_NORMAL 0
-#define EDIT_KEY_EMULATION_EMACS  1
-#define EDIT_KEY_EMULATION_USER   2
-
 #define REDRAW_LINE          (1 << 0)
 #define REDRAW_LINE_ABOVE    (1 << 1)
 #define REDRAW_LINE_BELOW    (1 << 2)
@@ -60,7 +50,7 @@
 #define REDRAW_COMPLETELY    (1 << 8)
 
 #define EDIT_TEXT_HORIZONTAL_OFFSET 0
-#define EDIT_TEXT_VERTICAL_OFFSET   1
+#define EDIT_TEXT_VERTICAL_OFFSET   0
 
 #define EDIT_RIGHT_EXTREME option_edit_right_extreme
 #define EDIT_LEFT_EXTREME option_edit_left_extreme
@@ -129,14 +119,6 @@
 #define edit_query_dialog2(h,t,a,b) query_dialog (h, t, D_NORMAL, 2, a, b)
 #define edit_query_dialog3(h,t,a,b,c) query_dialog (h, t, D_NORMAL, 3, a, b, c)
 
-#ifndef MAX_PATH_LEN
-#ifdef PATH_MAX
-#define MAX_PATH_LEN PATH_MAX
-#else
-#define MAX_PATH_LEN 1024
-#endif
-#endif
-
 /*** enums ***************************************************************************************/
 
 /* type for file which is currently being edited */
@@ -223,10 +205,7 @@ void edit_init_menu (struct WMenuBar *menubar);
 void menu_save_mode_cmd (void);
 int edit_translate_key (WEdit * edit, long x_key, int *cmd, int *ch);
 int edit_get_byte (WEdit * edit, long byte_index);
-char *edit_get_byte_ptr (WEdit * edit, long byte_index);
-char *edit_get_buf_ptr (WEdit * edit, long byte_index);
 int edit_get_utf (WEdit * edit, long byte_index, int *char_width);
-int edit_get_prev_utf (WEdit * edit, long byte_index, int *char_width);
 long edit_count_lines (WEdit * edit, long current, long upto);
 long edit_move_forward (WEdit * edit, long current, long lines, long upto);
 long edit_move_forward3 (WEdit * edit, long current, int cols, long upto);
@@ -267,7 +246,7 @@ long edit_write_stream (WEdit * edit, FILE * f);
 char *edit_get_write_filter (const char *writename, const char *filename);
 int edit_save_confirm_cmd (WEdit * edit);
 int edit_save_as_cmd (WEdit * edit);
-WEdit *edit_init (WEdit * edit, int lines, int columns, const char *filename, long line);
+WEdit *edit_init (WEdit * edit, int y, int x, int lines, int cols, const char *filename, long line);
 int edit_clean (WEdit * edit);
 gboolean edit_ok_to_exit (WEdit * edit);
 int edit_renew (WEdit * edit);
