@@ -78,8 +78,8 @@
 
 #include "fileopctx.h"
 #include "file.h"               /* file operation routines */
-#include "find.h"               /* do_find() */
-#include "hotlist.h"            /* hotlist_cmd() */
+#include "find.h"               /* find_file() */
+#include "hotlist.h"            /* hotlist_show() */
 #include "tree.h"               /* tree_chdir() */
 #include "midnight.h"           /* change_panel() */
 #include "usermenu.h"           /* MC_GLOBAL_MENU */
@@ -933,7 +933,7 @@ delete_cmd_local (void)
 void
 find_cmd (void)
 {
-    do_find ();
+    find_file ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1568,6 +1568,9 @@ save_setup_cmd (void)
     if (save_setup (TRUE, TRUE))
         message (D_NORMAL, _("Setup"), _("Setup saved to ~/%s"),
                  MC_USERCONF_DIR PATH_SEP_STR MC_CONFIG_FILE);
+    else
+        message (D_ERROR, _("Setup"), _("Unable to save setup to ~/%s"),
+                 MC_USERCONF_DIR PATH_SEP_STR MC_CONFIG_FILE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
