@@ -2240,7 +2240,13 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
     /* Update panel contents to avoid actions on deleted files */
     if (!panel->is_panelized)
     {
-        update_panels (UP_RELOAD, UP_KEEPSEL);
+        panel_update_flags_t flags = UP_RELOAD;
+
+        /* don't update panelized panel */
+        if (get_other_type () == view_listing && other_panel->is_panelized)
+            flags |= UP_ONLY_CURRENT;
+
+        update_panels (flags, UP_KEEPSEL);
         repaint_screen ();
     }
 
@@ -2262,8 +2268,6 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
     }
 
     ctx = file_op_context_new (operation);
-    tctx = file_op_total_context_new ();
-    gettimeofday (&(tctx->transfer_start), (struct timezone *) NULL);
 
     /* Show confirmation dialog */
     if (operation != OP_DELETE)
@@ -2294,11 +2298,11 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
             /* just copy */
             dest_dir_ = g_strdup (dest_dir);
         }
+
         if (dest_dir_ == NULL)
         {
-            file_op_total_context_destroy (tctx);
-            file_op_context_destroy (ctx);
-            return FALSE;
+            ret_val = FALSE;
+            goto ret_fast;
         }
 
         /* Generate confirmation prompt */
@@ -2313,10 +2317,9 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
 
         if (dest == NULL || dest[0] == '\0')
         {
-            file_op_total_context_destroy (tctx);
-            file_op_context_destroy (ctx);
             g_free (dest);
-            return FALSE;
+            ret_val = FALSE;
+            goto ret_fast;
         }
     }
     else if (confirm_delete)
@@ -2345,12 +2348,14 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
 
         if (i != 0)
         {
-            file_op_total_context_destroy (tctx);
-            file_op_context_destroy (ctx);
-            return FALSE;
+            ret_val = FALSE;
+            goto ret_fast;
         }
     }
 
+    tctx = file_op_total_context_new ();
+    gettimeofday (&tctx->transfer_start, (struct timezone *) NULL);
+
     {
         filegui_dialog_type_t dialog_type;
 
@@ -2638,8 +2643,9 @@ panel_operate (void *source_panel, FileOperation operation, gboolean force_singl
     }
 #endif /* WITH_BACKGROUND */
 
-    file_op_context_destroy (ctx);
     file_op_total_context_destroy (tctx);
+  ret_fast:
+    file_op_context_destroy (ctx);
 
     return ret_val;
 }
