@@ -120,7 +120,7 @@ int classic_progressbar = 1;
 typedef enum {
     MSDOS_SUPER_MAGIC     = 0x4d44,
     NTFS_SB_MAGIC         = 0x5346544e,
-    NTFS_3G_MAGIC         = 0x65735546,
+    FUSE_MAGIC         = 0x65735546,
     PROC_SUPER_MAGIC      = 0x9fa0,
     SMB_SUPER_MAGIC       = 0x517B,
     NCP_SUPER_MAGIC       = 0x564c,
@@ -174,9 +174,6 @@ typedef struct
 
 /*** file scope variables ************************************************************************/
 
-/* Used to save the hint line */
-static int last_hint_line;
-
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -197,12 +194,13 @@ filegui__check_attrs_on_fs (const char *fs_path)
     {
     case MSDOS_SUPER_MAGIC:
     case NTFS_SB_MAGIC:
-    case NTFS_3G_MAGIC:
     case PROC_SUPER_MAGIC:
     case SMB_SUPER_MAGIC:
     case NCP_SUPER_MAGIC:
     case USBDEVICE_SUPER_MAGIC:
         return FALSE;
+    default:
+        break;
     }
 #elif defined(HAVE_STRUCT_STATFS_F_FSTYPENAME) \
       || defined(HAVE_STRUCT_STATVFS_F_FSTYPENAME)
@@ -575,10 +573,6 @@ file_op_context_create_ui_without_init (FileOpContext * ctx, gboolean with_eta,
         create_dlg (TRUE, 0, 0, WY - minus + 1 + total_reserve, dlg_width,
                     dialog_colors, NULL, NULL, op_names[ctx->operation], DLG_CENTER | DLG_REVERSE);
 
-    last_hint_line = the_hint->widget.y;
-    if ((ui->op_dlg->y + ui->op_dlg->lines) > last_hint_line)
-        the_hint->widget.y = ui->op_dlg->y + ui->op_dlg->lines + 1;
-
     add_widget (ui->op_dlg,
                 button_new (WY - minus - 2 + total_reserve,
                             dlg_width / 2 + 1, FILE_ABORT,
@@ -657,8 +651,6 @@ file_op_context_destroy_ui (FileOpContext * ctx)
         g_free (ui);
         ctx->ui = NULL;
     }
-
-    the_hint->widget.y = last_hint_line;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -852,7 +844,7 @@ file_progress_show_deleting (FileOpContext * ctx, const char *s)
 FileProgressStatus
 file_progress_real_query_replace (FileOpContext * ctx,
                                   enum OperationMode mode, const char *destname,
-                                  struct stat * _s_stat, struct stat * _d_stat)
+                                  struct stat *_s_stat, struct stat *_d_stat)
 {
     FileOpContextUI *ui;
 
@@ -911,8 +903,7 @@ file_progress_real_query_replace (FileOpContext * ctx,
 char *
 file_mask_dialog (FileOpContext * ctx, FileOperation operation,
                   gboolean only_one,
-                  const char *format, const void *text,
-                  const char *def_text, gboolean * do_bg)
+                  const char *format, const void *text, const char *def_text, gboolean * do_bg)
 {
     const size_t FMDY = 13;
     const size_t FMDX = 68;
