@@ -37,14 +37,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <unistd.h>
 #include <pwd.h>                /* for username in xterm title */
 
 #include "lib/global.h"
 
 #include "lib/tty/tty.h"
-#include "lib/tty/key.h"        /* For init_key() */
-#include "lib/tty/mouse.h"
+#include "lib/tty/key.h"        /* KEY_M_* masks */
 #include "lib/tty/win.h"        /* xterm_flag */
 #include "lib/skin.h"
 #include "lib/util.h"
@@ -193,35 +191,35 @@ create_panel_menu (void)
 {
     GList *entries = NULL;
 
-    entries = g_list_append (entries, menu_entry_create (_("File listin&g"), CK_ListingCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Quick view"), CK_QuickViewCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Info"), CK_InfoCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Tree"), CK_TreeCmd));
-    entries = g_list_append (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("File listin&g"), CK_ListingCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Quick view"), CK_QuickViewCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Info"), CK_InfoCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Tree"), CK_TreeCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
     entries =
-        g_list_append (entries, menu_entry_create (_("&Listing mode..."), CK_ChangeListingCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Sort order..."), CK_Sort));
-    entries = g_list_append (entries, menu_entry_create (_("&Filter..."), CK_FilterCmd));
+        g_list_prepend (entries, menu_entry_create (_("&Listing mode..."), CK_ChangeListingCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Sort order..."), CK_Sort));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Filter..."), CK_FilterCmd));
 #ifdef HAVE_CHARSET
     entries =
-        g_list_append (entries, menu_entry_create (_("&Encoding..."), CK_PanelSetPanelEncoding));
+        g_list_prepend (entries, menu_entry_create (_("&Encoding..."), CK_PanelSetPanelEncoding));
 #endif
 #ifdef ENABLE_VFS_NET
-    entries = g_list_append (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_separator_create ());
 #ifdef ENABLE_VFS_FTP
-    entries = g_list_append (entries, menu_entry_create (_("FT&P link..."), CK_FtplinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("FT&P link..."), CK_FtplinkCmd));
 #endif
 #ifdef ENABLE_VFS_FISH
-    entries = g_list_append (entries, menu_entry_create (_("S&hell link..."), CK_FishlinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("S&hell link..."), CK_FishlinkCmd));
 #endif
 #ifdef ENABLE_VFS_SMB
-    entries = g_list_append (entries, menu_entry_create (_("SM&B link..."), CK_SmblinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("SM&B link..."), CK_SmblinkCmd));
 #endif
 #endif /* ENABLE_VFS_NET */
-    entries = g_list_append (entries, menu_separator_create ());
-    entries = g_list_append (entries, menu_entry_create (_("&Rescan"), CK_RereadCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("&Rescan"), CK_RereadCmd));
 
-    return entries;
+    return g_list_reverse (entries);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -231,34 +229,34 @@ create_file_menu (void)
 {
     GList *entries = NULL;
 
-    entries = g_list_append (entries, menu_entry_create (_("&View"), CK_ViewCmd));
-    entries = g_list_append (entries, menu_entry_create (_("Vie&w file..."), CK_ViewFileCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Filtered view"), CK_FilteredViewCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Edit"), CK_EditCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Copy"), CK_CopyCmd));
-    entries = g_list_append (entries, menu_entry_create (_("C&hmod"), CK_ChmodCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Link"), CK_LinkCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Symlink"), CK_SymlinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&View"), CK_ViewCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("Vie&w file..."), CK_ViewFileCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Filtered view"), CK_FilteredViewCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Edit"), CK_EditCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Copy"), CK_CopyCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("C&hmod"), CK_ChmodCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Link"), CK_LinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Symlink"), CK_SymlinkCmd));
     entries =
-        g_list_append (entries, menu_entry_create (_("Relative symlin&k"), CK_RelativeSymlinkCmd));
-    entries = g_list_append (entries, menu_entry_create (_("Edit s&ymlink"), CK_EditSymlinkCmd));
-    entries = g_list_append (entries, menu_entry_create (_("Ch&own"), CK_ChownCmd));
+        g_list_prepend (entries, menu_entry_create (_("Relative symlin&k"), CK_RelativeSymlinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("Edit s&ymlink"), CK_EditSymlinkCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("Ch&own"), CK_ChownCmd));
     entries =
-        g_list_append (entries, menu_entry_create (_("&Advanced chown"), CK_ChownAdvancedCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Rename/Move"), CK_RenameCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Mkdir"), CK_MkdirCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Delete"), CK_DeleteCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Quick cd"), CK_QuickCdCmd));
-    entries = g_list_append (entries, menu_separator_create ());
-    entries = g_list_append (entries, menu_entry_create (_("Select &group"), CK_SelectCmd));
-    entries = g_list_append (entries, menu_entry_create (_("U&nselect group"), CK_UnselectCmd));
+        g_list_prepend (entries, menu_entry_create (_("&Advanced chown"), CK_ChownAdvancedCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Rename/Move"), CK_RenameCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Mkdir"), CK_MkdirCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Delete"), CK_DeleteCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Quick cd"), CK_QuickCdCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("Select &group"), CK_SelectCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("U&nselect group"), CK_UnselectCmd));
     entries =
-        g_list_append (entries,
-                       menu_entry_create (_("Reverse selec&tion"), CK_ReverseSelectionCmd));
-    entries = g_list_append (entries, menu_separator_create ());
-    entries = g_list_append (entries, menu_entry_create (_("E&xit"), CK_QuitCmd));
+        g_list_prepend (entries,
+                        menu_entry_create (_("Reverse selec&tion"), CK_ReverseSelectionCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("E&xit"), CK_QuitCmd));
 
-    return entries;
+    return g_list_reverse (entries);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -272,54 +270,55 @@ create_command_menu (void)
      */
     GList *entries = NULL;
 
-    entries = g_list_append (entries, menu_entry_create (_("&User menu"), CK_UserMenuCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Directory tree"), CK_TreeBoxCmd));
-    entries = g_list_append (entries, menu_entry_create (_("&Find file"), CK_FindCmd));
-    entries = g_list_append (entries, menu_entry_create (_("S&wap panels"), CK_SwapCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&User menu"), CK_UserMenuCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Directory tree"), CK_TreeBoxCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Find file"), CK_FindCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("S&wap panels"), CK_SwapCmd));
     entries =
-        g_list_append (entries, menu_entry_create (_("Switch &panels on/off"), CK_ShowCommandLine));
+        g_list_prepend (entries,
+                        menu_entry_create (_("Switch &panels on/off"), CK_ShowCommandLine));
     entries =
-        g_list_append (entries, menu_entry_create (_("&Compare directories"), CK_CompareDirsCmd));
+        g_list_prepend (entries, menu_entry_create (_("&Compare directories"), CK_CompareDirsCmd));
 #ifdef USE_DIFF_VIEW
-    entries = g_list_append (entries, menu_entry_create (_("&View diff files"), CK_DiffViewCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&View diff files"), CK_DiffViewCmd));
 #endif
     entries =
-        g_list_append (entries, menu_entry_create (_("E&xternal panelize"), CK_ExternalPanelize));
+        g_list_prepend (entries, menu_entry_create (_("E&xternal panelize"), CK_ExternalPanelize));
     entries =
-        g_list_append (entries,
-                       menu_entry_create (_("Show directory s&izes"), CK_SingleDirsizeCmd));
-    entries = g_list_append (entries, menu_separator_create ());
-    entries = g_list_append (entries, menu_entry_create (_("Command &history"), CK_HistoryCmd));
+        g_list_prepend (entries,
+                        menu_entry_create (_("Show directory s&izes"), CK_SingleDirsizeCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("Command &history"), CK_HistoryCmd));
     entries =
-        g_list_append (entries, menu_entry_create (_("Di&rectory hotlist"), CK_QuickChdirCmd));
+        g_list_prepend (entries, menu_entry_create (_("Di&rectory hotlist"), CK_QuickChdirCmd));
 #ifdef ENABLE_VFS
-    entries = g_list_append (entries, menu_entry_create (_("&Active VFS list"), CK_ReselectVfs));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Active VFS list"), CK_ReselectVfs));
 #endif
 #ifdef WITH_BACKGROUND
-    entries = g_list_append (entries, menu_entry_create (_("&Background jobs"), CK_JobsCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Background jobs"), CK_JobsCmd));
 #endif
-    entries = g_list_append (entries, menu_entry_create (_("Screen lis&t"), CK_DialogListCmd));
-    entries = g_list_append (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("Screen lis&t"), CK_DialogListCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
 #ifdef ENABLE_VFS_UNDELFS
     entries =
-        g_list_append (entries,
-                       menu_entry_create (_("&Undelete files (ext2fs only)"), CK_UndeleteCmd));
+        g_list_prepend (entries,
+                        menu_entry_create (_("&Undelete files (ext2fs only)"), CK_UndeleteCmd));
 #endif
 #ifdef LISTMODE_EDITOR
     entries =
-        g_list_append (entries, menu_entry_create (_("&Listing format edit"), CK_ListmodeCmd));
+        g_list_prepend (entries, menu_entry_create (_("&Listing format edit"), CK_ListmodeCmd));
 #endif
 #if defined (ENABLE_VFS_UNDELFS) || defined (LISTMODE_EDITOR)
-    entries = g_list_append (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_separator_create ());
 #endif
     entries =
-        g_list_append (entries, menu_entry_create (_("Edit &extension file"), CK_EditExtFileCmd));
-    entries = g_list_append (entries, menu_entry_create (_("Edit &menu file"), CK_EditMcMenuCmd));
+        g_list_prepend (entries, menu_entry_create (_("Edit &extension file"), CK_EditExtFileCmd));
+    entries = g_list_prepend (entries, menu_entry_create (_("Edit &menu file"), CK_EditMcMenuCmd));
     entries =
-        g_list_append (entries,
-                       menu_entry_create (_("Edit hi&ghlighting group file"), CK_EditFhlFileCmd));
+        g_list_prepend (entries,
+                        menu_entry_create (_("Edit hi&ghlighting group file"), CK_EditFhlFileCmd));
 
-    return entries;
+    return g_list_reverse (entries);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -329,20 +328,21 @@ create_options_menu (void)
 {
     GList *entries = NULL;
 
-    entries = g_list_append (entries, menu_entry_create (_("&Configuration..."), CK_ConfigureBox));
-    entries = g_list_append (entries, menu_entry_create (_("&Layout..."), CK_LayoutBox));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Configuration..."), CK_ConfigureBox));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Layout..."), CK_LayoutBox));
     entries =
-        g_list_append (entries, menu_entry_create (_("&Panel options..."), CK_PanelOptionsBox));
-    entries = g_list_append (entries, menu_entry_create (_("C&onfirmation..."), CK_ConfirmBox));
-    entries = g_list_append (entries, menu_entry_create (_("&Display bits..."), CK_DisplayBitsBox));
-    entries = g_list_append (entries, menu_entry_create (_("Learn &keys..."), CK_LearnKeys));
+        g_list_prepend (entries, menu_entry_create (_("&Panel options..."), CK_PanelOptionsBox));
+    entries = g_list_prepend (entries, menu_entry_create (_("C&onfirmation..."), CK_ConfirmBox));
+    entries =
+        g_list_prepend (entries, menu_entry_create (_("&Display bits..."), CK_DisplayBitsBox));
+    entries = g_list_prepend (entries, menu_entry_create (_("Learn &keys..."), CK_LearnKeys));
 #ifdef ENABLE_VFS
-    entries = g_list_append (entries, menu_entry_create (_("&Virtual FS..."), CK_ConfigureVfs));
+    entries = g_list_prepend (entries, menu_entry_create (_("&Virtual FS..."), CK_ConfigureVfs));
 #endif
-    entries = g_list_append (entries, menu_separator_create ());
-    entries = g_list_append (entries, menu_entry_create (_("&Save setup"), CK_SaveSetupCmd));
+    entries = g_list_prepend (entries, menu_separator_create ());
+    entries = g_list_prepend (entries, menu_entry_create (_("&Save setup"), CK_SaveSetupCmd));
 
-    return entries;
+    return g_list_reverse (entries);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -718,69 +718,6 @@ ctl_x_cmd (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-init_xterm_support (void)
-{
-    const char *termvalue;
-
-    termvalue = getenv ("TERM");
-    if (!termvalue || !(*termvalue))
-    {
-        fputs (_("The TERM environment variable is unset!\n"), stderr);
-        exit (EXIT_FAILURE);
-    }
-
-    /* Check mouse capabilities */
-    xmouse_seq = tty_tgetstr ("Km");
-
-    if (strcmp (termvalue, "cygwin") == 0)
-    {
-        mc_args__force_xterm = 1;
-        use_mouse_p = MOUSE_DISABLED;
-    }
-
-    if (mc_args__force_xterm || strncmp (termvalue, "xterm", 5) == 0
-        || strncmp (termvalue, "konsole", 7) == 0
-        || strncmp (termvalue, "rxvt", 4) == 0
-        || strcmp (termvalue, "Eterm") == 0 || strcmp (termvalue, "dtterm") == 0)
-    {
-        xterm_flag = 1;
-
-#ifdef HAVE_SLANG
-        /* For 8-bit locales, NCurses handles 154 (0x9A) symbol properly, while S-Lang
-         * requires SLsmg_Display_Eight_Bit >= 154 (OR manual filtering if xterm display
-         * detected - but checking TERM would fail under screen, OR running xterm
-         * with allowC1Printable).
-         */
-        tty_display_8bit (FALSE);
-#endif
-
-        /* Default to the standard xterm sequence */
-        if (!xmouse_seq)
-        {
-            xmouse_seq = ESC_STR "[M";
-        }
-
-        /* Enable mouse unless explicitly disabled by --nomouse */
-        if (use_mouse_p != MOUSE_DISABLED)
-        {
-            const char *color_term = getenv ("COLORTERM");
-            if (strncmp (termvalue, "rxvt", 4) == 0 ||
-                (color_term != NULL && strncmp (color_term, "rxvt", 4) == 0) ||
-                strcmp (termvalue, "Eterm") == 0)
-            {
-                use_mouse_p = MOUSE_XTERM_NORMAL_TRACKING;
-            }
-            else
-            {
-                use_mouse_p = MOUSE_XTERM_BUTTON_EVENT_TRACKING;
-            }
-        }
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static void
 setup_mc (void)
 {
@@ -799,9 +736,6 @@ setup_mc (void)
 
     if ((tty_baudrate () < 9600) || tty_is_slow ())
         verbose = 0;
-
-    init_xterm_support ();
-    init_mouse ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -819,23 +753,9 @@ setup_dummy_mc (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-done_screen (void)
-{
-    if ((quit & SUBSHELL_EXIT) == 0)
-        clr_scr ();
-    tty_reset_shell_mode ();
-    tty_noraw_mode ();
-    tty_keypad (FALSE);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static void
 done_mc (void)
 {
-    disable_mouse ();
-
     /* Setup shutdown
      *
      * We sync the profiles since the hotlist may have changed, while
@@ -843,7 +763,6 @@ done_mc (void)
      */
 
     save_setup (auto_save_setup, panels_options.auto_save_setup);
-    done_screen ();
 
     vfs_stamp_path (vfs_get_current_dir ());
 
@@ -1640,6 +1559,9 @@ do_nc (void)
 #ifdef USE_INTERNAL_EDIT
     edit_stack_free ();
 #endif
+
+    if ((quit & SUBSHELL_EXIT) == 0)
+        clr_scr ();
 }
 
 /* --------------------------------------------------------------------------------------------- */
