@@ -327,33 +327,18 @@ do_cd (const char *new_dir, enum cd_enum exact)
 /* --------------------------------------------------------------------------------------------- */
 
 #ifdef HAVE_SUBSHELL_SUPPORT
-int
-load_prompt (int fd, void *unused)
+gboolean
+do_load_prompt (void)
 {
-    (void) fd;
-    (void) unused;
+    gboolean ret = FALSE;
 
     if (!read_subshell_prompt ())
-        return 0;
+        return ret;
 
     /* Don't actually change the prompt if it's invisible */
     if (((Dlg_head *) top_dlg->data == midnight_dlg) && command_prompt)
     {
-        char *tmp_prompt;
-        int prompt_len;
-
-        tmp_prompt = strip_ctrl_codes (subshell_prompt);
-        prompt_len = str_term_width1 (tmp_prompt);
-
-        /* Check for prompts too big */
-        if (COLS > 8 && prompt_len > COLS - 8)
-        {
-            prompt_len = COLS - 8;
-            tmp_prompt[prompt_len] = '\0';
-        }
-        mc_prompt = tmp_prompt;
-        label_set_text (the_prompt, mc_prompt);
-        input_set_origin ((WInput *) cmdline, prompt_len, COLS - prompt_len);
+        setup_cmdline ();
 
         /* since the prompt has changed, and we are called from one of the
          * tty_get_event channels, the prompt updating does not take place
@@ -361,8 +346,21 @@ load_prompt (int fd, void *unused)
          */
         update_cursor (midnight_dlg);
         mc_refresh ();
+        ret = TRUE;
     }
     update_subshell_prompt = TRUE;
+    return ret;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+int
+load_prompt (int fd, void *unused)
+{
+    (void) fd;
+    (void) unused;
+
+    do_load_prompt ();
     return 0;
 }
 #endif /* HAVE_SUBSHELL_SUPPORT */
@@ -440,6 +438,13 @@ main (int argc, char *argv[])
     if (!mc_args_handle (argc, argv, "mc"))
         exit (EXIT_FAILURE);
 
+    /* check terminal type
+     * $TEMR must be set and not empty
+     * xterm_flag is used in init_key() and tty_init()
+     * Do this after mc_args_handle() where mc_args__force_xterm is set up.
+     */
+    xterm_flag = tty_check_term (mc_args__force_xterm);
+
     /* NOTE: This has to be called before tty_init or whatever routine
        calls any define_sequence */
     init_key ();
@@ -464,7 +469,8 @@ main (int argc, char *argv[])
 
     /* Must be done before init_subshell, to set up the terminal size: */
     /* FIXME: Should be removed and LINES and COLS computed on subshell */
-    tty_init ((gboolean) mc_args__slow_terminal, (gboolean) mc_args__ugly_line_drawing);
+    tty_init (mc_args__slow_terminal, mc_args__ugly_line_drawing, !mc_args__nomouse,
+              (gboolean) xterm_flag);
 
     load_setup ();
 
