@@ -130,8 +130,8 @@ mcview_toggle_hex_mode (mcview_t * view)
     }
     else
     {
-        view->dpy_start = view->hex_cursor;
-        mcview_moveto_bol (view);
+        view->dpy_start = mcview_bol (view, view->hex_cursor, 0);
+        view->hex_cursor = view->dpy_start;
         widget_want_cursor (view->widget, 0);
     }
     mcview_altered_hex_mode = 1;
@@ -237,7 +237,9 @@ mcview_done (mcview_t * view)
     {
         char *canon_fname;
         canon_fname = vfs_canon (view->filename);
-        save_file_position (canon_fname, -1, 0, view->dpy_start, view->saved_bookmarks);
+        save_file_position (canon_fname, -1, 0,
+                            view->hex_mode ? view->hex_cursor : view->dpy_start,
+                            view->saved_bookmarks);
         view->saved_bookmarks = NULL;
         g_free (canon_fname);
     }
